//
// Created by root on 6/6/19.
//

#ifndef IOT_DEMO_MQTT_H
#define IOT_DEMO_MQTT_H

#include <stdbool.h>
#include <stdint.h>
#include "cJSON.h"
#include "iot_client_error.h"

typedef enum IotUploadType {
    IOT_UPLOAD_NONE = 0, /** @ref 默认 */
    IOT_UPLOAD_PROPERTIES, /** @ref 属性 */
    IOT_UPLOAD_EVENTS, /** @ref 事件 */
    IOT_UPLOAD_SERVICES /** @ref 服务 */
} IotUploadType_t;

typedef enum IotDisconnectReason {
    IOT_DISCONNECT_CALLED,   /**< @ref mqtt_function_disconnect was invoked. */
    IOT_BAD_PACKET_RECEIVED, /**< An invalid packet was received from the network. */
    IOT_KEEP_ALIVE_TIMEOUT   /**< Keep-alive response was not received within @ref IOT_MQTT_RESPONSE_WAIT_MS. */
} IotDisconnectReason_t;

typedef struct IotArguments {
    bool isAutoReconnect; /**< @brief  Whether to automatically reconnect after disconnecting*/
    bool securedConnection; /**< @brief Whether to secure the network connection with TLS. */
    char *pHostName; /**< @brief The remote host name that the demo will connect to. */
    uint16_t port;          /**< @brief The resmote host port that the demo will connect to. */

    char *pProductKey;
    char *pDeviceName;
    char *pDeviceSecret;

    /* These credentials are only used if securedConnection is true. */
    const char *pRootCaPath;     /**< @brief The path to the server root certificate to use for the connection. */
    const char *pClientCertPath; /**< @brief The path to the client certificate to use for the connection. */
    const char *pPrivateKeyPath; /**< @brief the path to the private key that matches the client certificate. */
    const char *pUserName;       /**< @brief The username for authenticating to the MQTT broker. */
    const char *pPassword;       /**< @brief The password for authenticating to the MQTT broker. */

    const char *pIdentifier;     /**< @brief Client identifier or Thing Name to use for demo. */
} IotArguments_t;

typedef struct IotClientCallback {

    /** @brief 连接成功回调 */
    int (*cbConnect)();

    /** @brief 连接失败回调 **/
    int (*cbConnectFailure)(IotClientError_t error, const char *errorMsg);

    /** @brief 断开了解回调 */
    int (*cbDisconnect)(IotDisconnectReason_t *reason);

    /** @brief 获取属性回调 */
    int (*cbGetProperties)(cJSON *inParams, cJSON *outParams);

    /** @brief 设置属性回调 */
    int (*cbSetProperties)(cJSON *inParams, char **msg);

    /** @brief 调用服务回调 */
    int (*cbCallServices)(cJSON *inParams, cJSON *outParams, char **msg);

    /** @brief 消息发送状态回调 */
    int (*cbUploadStatus)(const char *mid, IotUploadType_t type, bool status);

    /** @brief 上报属性和事件回调 */
    int (*cbUploadResponses)(cJSON *inParams, const char *mid, IotUploadType_t type);
} IotClientCallback_t;

typedef struct IotClient {
    IotClientCallback_t cb;
    IotArguments_t arguments;
} IotClient_t;

IotClient_t iotClient;

/** @brief 设置本地可读写的目录路径
 * @param localPath 目录路径，例如：.、/data/log等 */
int IotClient_LocalPath(char *localPath);

/** @brief 设置本地日志开关
 * @param isOpen 日志开关 */
int IotClient_OpenLog(bool isOpen);

/** @brief 设置云端日志
 *  @param isFileMode：文件模式
 *  @param logSize 日志大小上限，默认1KB
 *  @param logLine 日志行数上限，默认10行 */
int IotClient_InitCloudLog(bool isFileMode, int logSize, int logLine);

/**
 *
 * @param cb
 * @param arguments
 * @return
 */
int IotClient_Init(IotClientCallback_t cb, IotArguments_t arguments);

/** @brief 向云端发起连接 */
int IotClient_Connect();

/** @brief 主动与云端断开连接 */
int IotClient_Disconnect();

/** @brief 上报事件
 * @param params 事件相关参数
 * @param event 事件标识
 * @param outMid 事件消息标识 */
int IotClient_UploadEvents(cJSON *params, char *event, char **outMid);

/** @brief 上报属性
 * @param params 上报属性相关参数
 * @param outMid 属性消息标识 */
int IotClient_UploadProperties(cJSON *params, char **outMid);

#endif //IOT_DEMO_MQTT_H