//
// OTA服务
// Created by 郑家柱 on 2019-03-22.
//

#ifndef IOT_OTA_H
#define IOT_OTA_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdbool.h>
#include <cJSON.h>

typedef enum IotOtaStatus {
    IOT_OTA_STATUS_BURN_FAIL = -3, /** @brief 烧录失败 */
    IOT_OTA_STATUS_CHECK_FAIL = -2,  /** @brief 校验失败 */
    IOT_OTA_STATUS_DOWNLOAD_FAIL = -1,  /** @brief 下载失败 */
    IOT_OTA_STATUS_WAIT = 0, /** @brief 待升级 */
    IOT_OTA_STATUS_ON = 1, /** @brief 升级中 */
    IOT_OTA_STATUS_SUCCESS = 2,  /** @brief 升级成功 */
    IOT_OTA_STATUS_FAIL = 3,  /** @brief 升级失败 */
} IotOtaStatus_t;

typedef enum IotOtaType {
    IOT_OTA_TYPE_FULL = 0, /** @brief 整包升级 */
    IOT_OTA_TYPE_DIFF = 1, /** @brief 差分升级 */
} IotOtaType_t;

typedef struct IotOtaCallback {

    /**
     * @brief 收到固件升级推送的时候回调
     * @param version 固件版本号
     * @return true:下载固件 false:放弃升级
     */
    bool (*cbReceivedUpgrade)(char *version);

    /**
     * @brief 固件下载进度回调
     * @param progress 进度值
     */
    void (*cbUpgradeProgress)(float progress);

    /**
     * @brief 下载失败或者校验失败回调
     * @param status 失败状态
     */
    void (*cbUpgradeStatus)(IotOtaStatus_t status);

    /**
     * @brief 固件下载完成回调
     * @param type 固件类型 整包或者差分
     * @param name 固件名称
     * @param version 固件版本号
     * @param diff 差分升级的待升级版本号
     * @param path 固件下载后的路径
     */
    void (*cbNoticeUpgrade)(IotOtaType_t type, char *name, char *version, char *diff, char *path);

    /**
     * @brief 固件版本上报回调
     * @param result true: 成功 false:失败
     * @param msg 描述信息
     */
    void (*cbVersionResponse)(bool result, char *msg);

    /**
     * @brief 上报状态回调
     * @param result true: 成功 false:失败
     * @param msg 描述信息
     */
    void (*cbStatusResponse)(bool result, char *msg);
} IotOtaCallback_t;

typedef struct IotOta {
    bool isOpenOTA; /** @ref 是否开启OTA */
    bool isResume; /** @brief 是否开启断点续传 */
    char *firmwareDir; /** @ref 固件存放的文件夹路径 */
    IotOtaCallback_t cb;
} IotOta_t;

/** @brief 初始化OTA服务
 *
 * @param isOpen 是否开启OTA服务
 * @param isResume 是否开启断点续传
 * @param path 存放固件的目录路径，例如：/data
 */
int IotOta_Init(bool isOpen, bool isResume, const char *path, IotOtaCallback_t cb);

/** @brief 上报固件版本
 *
 * @param version 当前固件版本
 */
int IotOta_UploadFirmwareVersion(char *version);

/** @brief 上报固件状态
 *
 * @param status 上报状态
 * @param desc 描述
 */
int IotOta_UploadUpgradeStatus(IotOtaStatus_t status, char *desc);

/** @brief 上报固件版本响应
 *
 * @param data 响应参数
 */
int IotOta_UploadFirmwareVersionResponse(cJSON *data);

/** @brief 上报固件状态响应
 *
 * @param data 响应参数
 */
int IotOta_UploadUpgradeStatusResponse(cJSON *data);

/** @brief 主动推送固件升级信息
 *
 * @param data 固件升级相关参数
 * @param msg 描述
 */
int IotOta_NoticeUpgradeFirmware(cJSON *data, char **msg);

#ifdef __cplusplus
}
#endif

#endif //IOT_OTA_H
