/*
 * Copyright (C) 2018 Amazon.com, Inc. or its affiliates.  All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/**
 * @file iot_demo.c
 * @brief Generic demo runner.
 */

/* The config header is always included first. */
#include <iot_client_config.h>

/* Standard includes. */
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>
#include <cJSON.h>
#include <iot_client.h>
#include <unistd.h>
#include <sys/time.h>
#include <iot_notice.h>
#include <iot_ota.h>
#include <iot_remote_config.h>
#include <stdio.h>

#include "pthread.h"

#define UUID_LEN 37

bool isExistUploadProperties = false;
bool isExistUploadEvents = false;
bool isOpenOta = false;
bool isOpenRemoteConfig = false;

char *pProductKey;
char *pDeviceName;
char *pDeviceSecret;
char *pVersion;

void create_upload_properties();

void create_upload_events();

void create_open_ota();

void create_remote_config();

/*-----------------------------------------------------------*/

int cbConnect() {
//    create_upload_properties();
//    create_upload_events();
//    create_remote_config();
    create_open_ota();
    return 0;
}

int cbDisconnect(IotDisconnectReason_t *reason) {
    return 0;
}

int cbConnectFailure(IotClientError_t error, const char *errorMsg) {

    IotClient_Disconnect();
    return 0;
}

int cbGetProperties(cJSON *inParams, cJSON *outParams) {

    cJSON *params = inParams->child;
    if (!params) {
        return -1;
    }

    while (params) {
        char *key = params->valuestring;
        if (!key) {
            break;
        }

        if (!strcmp("PowerSwitch", key)) {
            cJSON_AddBoolToObject(outParams, key, true);
        } else if (!strcmp("WorkMode", key)) {
            cJSON_AddNumberToObject(outParams, key, 1);
        } else if (!strcmp("TargetTemperature", key)) {
            cJSON_AddNumberToObject(outParams, key, 26);
        } else if (!strcmp("CurrentTemperature", key)) {
            cJSON_AddNumberToObject(outParams, key, 28);
        } else if (!strcmp("WindSpeed", key)) {
            cJSON_AddNumberToObject(outParams, key, 2);
        } else if (!strcmp("SleepMode", key)) {
            cJSON_AddBoolToObject(outParams, key, false);
        } else if (!strcmp("SleepTime", key)) {
            cJSON_AddNumberToObject(outParams, key, 8);
        }
        params = params->next;
    }
    return 0;
}

int cbSetProperties(cJSON *inParams, char **msg) {

    cJSON *params = inParams->child;
    if (params == NULL) {
        return -1;
    }

    char *key = params->string;
    if (!strcmp("PowerSwitch", key)) {
        if (cJSON_False != params->type && cJSON_True != params->type) {
            return -1;
        }

        //IotLogInfo("setProperties PowerSwitch:%d", params->valueint);
    } else if (!strcmp("WorkMode", key)) {
        if (cJSON_Number != params->type) {
            return -1;
        }

        //IotLogInfo("setProperties WorkMode:%d", params->valueint);
    } else if (!strcmp("TargetTemperature", key)) {
        if (cJSON_Number != params->type) {
            return -1;
        }

        //IotLogInfo("setProperties TargetTemperature:%d", params->valuedouble);
    } else if (!strcmp("CurrentTemperature", key)) {
        if (cJSON_Number != params->type) {
            return -1;
        }

        //IotLogInfo("setProperties CurrentTemperature:%d", params->valuedouble);
    } else if (!strcmp("WindSpeed", key)) {
        if (cJSON_Number != params->type) {
            return -1;
        }

        //IotLogInfo("setProperties WindSpeed:%d", params->valueint);
    } else if (!strcmp("SleepMode", key)) {
        if (cJSON_Number != params->type) {
            return -1;
        }

        //IotLogInfo("setProperties SleepMode:%d", params->valueint);
    } else if (!strcmp("SleepTime", key)) {
        if (cJSON_Number != params->type) {
            return -1;
        }

        //IotLogInfo("setProperties SleepTime:%d", params->valueint);
    }

    *msg = "set properties ok";
    return 0;
}

int cbCallServices(cJSON *inParams, cJSON *outParams, char **msg) {

    char *identifier = inParams->child->valuestring;
    if (!identifier) {
        return -1;
    }

    if (!strcmp("AdjustTemperature", identifier)) {
        cJSON *paramsJson = cJSON_GetObjectItem(inParams, "params");
        if (!paramsJson) {
            return -1;
        }

        cJSON *tmpJson = paramsJson->child;

        while (tmpJson) {
            char *key = tmpJson->string;
            if (!key) {
                break;
            }
            //IotLogInfo("key1=%s", key);

            if (!strcmp("Up", key)) {
                if (tmpJson->type == cJSON_Number) {
                    //IotLogInfo("Up=%d", tmpJson->valueint);
                }
            } else if (!strcmp("Down", key)) {
                if (tmpJson->type == cJSON_Number) {
                    //IotLogInfo("Down=%d", tmpJson->valueint);
                }
            }

            tmpJson = tmpJson->next;
        }
    } else if (!strcmp("AdjustFanSpeed", identifier)) {
        cJSON *paramsJson = cJSON_GetObjectItem(inParams, "params");
        if (!paramsJson) {
            return -1;
        }

        cJSON *tmpJson = paramsJson->child;

        while (tmpJson) {
            char *key = tmpJson->string;
            if (!key) {
                break;
            }
            //IotLogInfo("key1=%s", key);

            if (!strcmp("Up", key)) {
                if (tmpJson->type == cJSON_Number) {
                    //IotLogInfo("Up=%d", tmpJson->valueint);
                }
            } else if (!strcmp("Down", key)) {
                if (tmpJson->type == cJSON_Number) {
                    //IotLogInfo("Down=%d", tmpJson->valueint);
                }
            }

            tmpJson = tmpJson->next;
        }
    }

    //IotLogInfo("identifier=%s", identifier);

    if (outParams) {
        cJSON_AddBoolToObject(outParams, "status", 1);
    }

    *msg = "call service ok";

    return 0;
}

int cbNoticeMessages(cJSON *inParams, cJSON *outParams, char **msg) {

    char *identifier = inParams->child->valuestring;
    if (!identifier) {
        return -1;
    }

    //IotLogInfo("identifier=%s", identifier);

    if (outParams) {
        cJSON_AddBoolToObject(outParams, "status", 1);
    }

    *msg = "notice messages ok";

    return 0;
}

int cbUploadStatus(const char *mid, IotUploadType_t type, bool status) {
    //IotLogInfo("upload message result mid: %s type:%d status:%d", mid, type, status);
    return 0;
}

// 客户端上传属性或者事件回调
int cbUploadResponses(cJSON *inParams, const char *mid, IotUploadType_t type) {

    //IotLogInfo("on_postValue type=%d", type);

    if (mid) {
        //IotLogInfo("on_postValue mid=%s", mid);
    }

    return 0;
}

bool cbReceivedUpgrade(char *version) {
    //IotLogDebug("received firmware upgrade");
    return true;
}

void cbUpgradeProgress(float progress) {
    //IotLogDebug("firmware progress:%f", progress);
}

void cbUpgradeStatus(IotOtaStatus_t status) {

}

// 通知OTA固件升级回调
void cbNoticeUpgrade(IotOtaType_t type, char *name, char *version, char *diff, char *path) {
    //IotLogDebug("notice firmware upgrade type:%d name:%s version:%s diff:%s path:%s", type, name, version, diff, path);
    IotOta_UploadUpgradeStatus(IOT_OTA_STATUS_ON, "正在升级");

    //sleep(5);
    //remove(path);
    sleep(5);
    IotOta_UploadFirmwareVersion(version);
}

void cbVersionResponse(bool result, char *msg) {
    //IotLogDebug("firmware upload version:%d desc:%s", result, msg);
}

void cbStatusResponse(bool result, char *msg) {
    //IotLogDebug("firmware upload status:%d desc:%s", result, msg);
}

void cbRemoteVersionResponse(bool result, char *msg) {
    //IotLogDebug("remote upload version:%d desc:%s", result, msg);
}

void cbRemoteStatusResponse(bool result, char *msg) {
    //IotLogDebug("remote upload status:%d desc:%s", result, msg);
}

int cbRemoteNotice(cJSON *config, int version, char **msg) {
    //IotLogDebug("remote config:%s version:%d", cJSON_PrintUnformatted(config), version);
    *msg = "remote config ok";

    IotRemoteConfig_UploadStatus(IOT_REMOTE_CONFIG_STATUS_SUCCESS, version, *msg);
    return 0;
}

/*-----------------------------------------------------------*/

void *upload_properties() {
    int rc = 0;
    do {
        struct timeval time;
        time.tv_sec = 10;
        time.tv_usec = 0;

        rc = select(0, NULL, NULL, NULL, &time);
        if (rc == 0) {

            cJSON *params = cJSON_CreateObject();
            cJSON_AddNumberToObject(params, "WorkMode", 1);

            char *mid = malloc(UUID_LEN);
            memset(mid, 0, UUID_LEN);
            int ret = IotClient_UploadProperties(params, &mid);
            //IotLogInfo("upload properties mid:%s", mid);
            if (ret != 0) {
                //IotLogInfo("post value error:%d", ret);
            }
            free(mid);
        }
    } while (rc == 0);
    return ((void *) 0);
}

// 定时上报设备事件
void *upload_events() {

    int rc = 0;
    do {
        struct timeval time;
        time.tv_sec = 10;
        time.tv_usec = 0;

        rc = select(0, NULL, NULL, NULL, &time);
        if (rc == 0) {

            cJSON *params = cJSON_CreateObject();
            cJSON_AddNumberToObject(params, "WorkMode", 1);

            char *mid = malloc(UUID_LEN);
            memset(mid, 0, UUID_LEN);
            int ret = IotClient_UploadEvents(params, "system.msg.transform.upload", &mid);
            //IotLogInfo("upload events mid:%s", mid);
            if (ret != 0) {
                //IotLogInfo("post value error:%d", ret);
            }
            free(mid);
        }
    } while (rc == 0);
    return ((void *) 0);
}

void create_upload_properties() {

    if (isExistUploadProperties == true) {
        return;
    }

    isExistUploadProperties = true;

    pthread_t timer_thread;
    int rc = pthread_create(&timer_thread, NULL, upload_properties, NULL);
    if (rc != 0) {
        //IotLogInfo("pthread create error:%d", rc);
    }
}

void create_upload_events() {
    if (isExistUploadEvents == true) {
        return;
    }

    isExistUploadEvents = true;

    // 创建线程
    pthread_t timer_thread;
    int rc = pthread_create(&timer_thread, NULL, upload_events, NULL);
    if (rc != 0) {
        //IotLogInfo("pthread create error:%d", rc);
    }
};

void create_open_ota() {

    if (isOpenOta == true) {
        return;
    }

    isOpenOta = true;

    IotOtaCallback_t callback = {
            .cbReceivedUpgrade = cbReceivedUpgrade,
            .cbUpgradeProgress = cbUpgradeProgress,
            .cbUpgradeStatus = cbUpgradeStatus,
            .cbNoticeUpgrade = cbNoticeUpgrade,
            .cbVersionResponse = cbVersionResponse,
            .cbStatusResponse = cbStatusResponse
    };

    // 开启OTA
    IotOta_Init(true, false, "./", callback);

    IotOta_UploadFirmwareVersion(pVersion);
};

void create_remote_config() {

    if (isOpenRemoteConfig == true) {
        return;
    }

    IotRemoteConfigCallback_t callback = {
            .cbRemoteNotice = cbRemoteNotice,
            .cbRemoteVersionResponse = cbRemoteVersionResponse,
            .cbRemoteStatusResponse = cbRemoteStatusResponse
    };

    isOpenRemoteConfig = true;

    IotRemoteConfig_Init(true, callback);

    IotRemoteConfig_UploadVersion(0);
};

/*-----------------------------------------------------------*/

int main(int argc, char **argv) {

    pProductKey = IOT_PRODUCT_KEY;
    pDeviceName = IOT_DEVICE_NAME;
    pDeviceSecret = IOT_DEVICE_SECRET;
    pVersion = "2.0.1";

    for (int i = 1; i < argc; i++) {

        if (i == 1) {
            pProductKey = argv[i];
        }

        if (i == 2) {
            pDeviceName = argv[i];
        }

        if (i == 3) {
            pDeviceSecret = argv[i];
        }

        if (i == 4) {
            pVersion = argv[i];
        }
    }

    IotArguments_t arguments;
    arguments.isAutoReconnect = true;
    arguments.pIdentifier = IOT_IDENTIFIER;
    arguments.securedConnection = IOT_SECURED_CONNECTION;
    arguments.pHostName = IOT_SERVER_HOST;
    arguments.port = IOT_SERVER_PORT;
    arguments.pProductKey = pProductKey;
    arguments.pDeviceName = pDeviceName;
    arguments.pDeviceSecret = pDeviceSecret;
    arguments.pRootCaPath = IOT_ROOT_CA;
    arguments.pClientCertPath = IOT_CLIENT_CERT;
    arguments.pPrivateKeyPath = IOT_PRIVATE_KEY;
    arguments.pUserName = "";
    arguments.pPassword = "";

    IotClientCallback_t callback;
    callback.cbConnect = cbConnect;
    callback.cbConnectFailure = cbConnectFailure;
    callback.cbDisconnect = cbDisconnect;
    callback.cbGetProperties = cbGetProperties;
    callback.cbSetProperties = cbSetProperties;
    callback.cbCallServices = cbCallServices;
    callback.cbUploadStatus = cbUploadStatus;
    callback.cbUploadResponses = cbUploadResponses;

    IotNoticeCallback_t callback1 = {.cbNoticeMessages = cbNoticeMessages};
    IotNotice_Init(callback1);

    IotClient_Init(callback, arguments);
    IotClient_LocalPath(".");
    IotClient_InitCloudLog(false, 2, 20);
    IotClient_Connect();

    while (true) {
        sleep(2);
    }
}

/*-----------------------------------------------------------*/
